import 'dart:async';

import 'package:background_geolocation_plugin/measure_state.dart';
import 'package:flutter/services.dart';

import 'location_item.dart';

/// Main plugin class
class BackgroundGeolocationPlugin {
  /// The communication channel used to comminicate with native code
  static const MethodChannel _channel =
      const MethodChannel('background_geolocation_plugin');

  /// Starts geolocation service in the backgrund and also make this service run in the foreground
  /// to prevent kill it by the operating system.
  /// If the service started properly, it automatically starts to
  /// retireve and store all incoming geolocation updates from GPS.
  static Future<String> startLocationTracking() async {
    final String res = await _channel.invokeMethod('startLocationTracking');
    return res;
  }

  /// Stops backgorund geolocation service.
  ///
  /// After succesfull exuction of this method, you are not able
  /// to retrieve saved locations from just stopped 'measure'.
  /// Before executing this method, make sure to get stored locations if needed.
  static Future<String> stopLocationTracking() async {
    final String res = await _channel.invokeMethod('stopLocationTracking');
    return res;
  }

  /// Stops saving incoming geolocations in backgound geolocation service
  static Future<String> pauseLocationTracking() async {
    final String res = await _channel.invokeMethod('pauseLocationTracking');
    return res;
  }

  /// Continue saving incoming geolocaitons in background geolocation service
  static Future<String> continueLocationTracking() async {
    final String res = await _channel.invokeMethod('continueLocationTracking');
    return res;
  }

  /// Returns state of the backgorund geolocation service
  static Future<MeasureState> getState() async {
    Map<dynamic, dynamic> state = await _channel.invokeMethod('getState');
    return MeasureState(Map<String, dynamic>.from(state));
  }

  /// Requests permissions needed by backgorund geolocation service if not yet granted.
  static Future<String> requestPermissions() async {
    String res = await _channel.invokeMethod('requestPermissions');
    return res;
  }

  /// Returns all saved locations so far, by the backgorund geolocation service
  static Future<List<LocationItem>> getAllLocations() async {
    try {
      List<dynamic> result = await _channel.invokeMethod('getAllLocations');
      return result
          .cast<Map<dynamic, dynamic>>()
          .map((item) => LocationItem(item.cast<String, dynamic>()))
          .toList();
    } on PlatformException catch (e) {
      print("error: " + e.message);
      return null;
    }
  }

  /// Returns all saved locations, for which time is bigger than provided.
  static Future<List<LocationItem>> getAllStoredLocationsWithTimeBiggerThan(
      int time) async {
    try {
      List<dynamic> result = await _channel
          .invokeMethod('getAllLocationsWithTimeBiggerThan', {"time": time});
      return result
          .cast<Map<dynamic, dynamic>>()
          .map((item) => LocationItem(item.cast<String, dynamic>()))
          .toList();
    } on PlatformException catch (e) {
      print("error: " + e.message);
      return null;
    }
  }
}
